//
use kameo::message::{Context, Message};

use crate::alerts::{messages::NotifyUser, webhook::send_webhook, AlertsActor};
use crate::prelude::*;

#[cfg(feature = "notify-desktop")]
use crate::alerts::desktop::send_desktop_notification;

#[cfg(feature = "notify-email")]
use crate::alerts::email::send_email;

impl Message<NotifyUser> for AlertsActor {
    type Reply = ();

    async fn handle(
        &mut self,
        msg: NotifyUser,
        _ctx: &mut Context<Self, Self::Reply>,
    ) -> Self::Reply {
        let Some(shout) = &self.shout else {
            debug!("No shout config available");
            return;
        };
        let net = self.net.clone();
        let shout = shout.clone();
        let Some(user_notification) = self.history.sendable(msg.peel()) else {
            return;
        };
        if let Some(webhook_config) = shout.webhook() {
            send_webhook(&user_notification, webhook_config, &net).await;
        }

        #[cfg(feature = "notify-email")]
        if let Some(email_config) = shout.email() {
            send_email(&user_notification, email_config);
        }

        #[cfg(feature = "notify-desktop")]
        if let Some(desktop) = shout.desktop() {
            if desktop {
                send_desktop_notification(&user_notification);
            }
        }
    }
}
