//
use secrecy::ExposeSecret as _;
use standardwebhooks::Webhook;

use crate::core::{git::UserNotification, server::OutboundWebhook};
use crate::prelude::*;

pub(super) async fn send_webhook(
    user_notification: &UserNotification,
    webhook_config: &OutboundWebhook,
    net: &kxio::net::Net,
) {
    let Ok(webhook) =
        Webhook::from_bytes(webhook_config.secret().expose_secret().as_bytes().into())
    else {
        warn!("Invalid notification configuration (signer) - can't sent notification");
        return;
    };
    do_send_webhook(user_notification, webhook, webhook_config, net).await;
}

async fn do_send_webhook(
    user_notification: &UserNotification,
    webhook: Webhook,
    webhook_config: &OutboundWebhook,
    net: &kxio::net::Net,
) {
    let message_id = format!("msg_{}", ulid::Ulid::new());
    let timestamp = time::OffsetDateTime::now_utc();
    let payload = user_notification.as_json(timestamp);
    let timestamp = timestamp.unix_timestamp();
    let to_sign = format!("{message_id}.{timestamp}.{payload}");
    info!(?to_sign, "");
    #[allow(clippy::expect_used)]
    let signature = webhook
        .sign(&message_id, timestamp, payload.to_string().as_ref())
        .expect("signature");
    info!(?signature, "");
    net.post(webhook_config.url())
        .body(payload.to_string())
        .header("webhook-id", message_id)
        .header("webhook-timestamp", timestamp.to_string())
        .header("webhook-signature", signature)
        .send()
        .await
        .map_or_else(
            |err| {
                warn!(?err, "sending webhook");
            },
            |_| (),
        );
}
