/// Identifier for the type of Forge
#[derive(
    Clone,
    Default,
    Debug,
    derive_more::From,
    PartialEq,
    Eq,
    PartialOrd,
    Ord,
    serde::Serialize,
    serde::Deserialize,
    Copy,
)]
pub enum ForgeType {
    #[cfg(feature = "forge-forgejo")]
    Forgejo,
    // Gitea,
    #[cfg(feature = "forge-github")]
    GitHub,
    // GitLab,
    // BitBucket,
    #[default]
    MockForge,
}
impl std::fmt::Display for ForgeType {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "{self:?}")
    }
}

#[cfg(test)]
mod tests {

    #[cfg(feature = "forge-forgejo")]
    mod forgejo {

        use crate::core::ForgeType;
        use crate::prelude::*;

        use rstest::rstest;

        #[rstest]
        #[case("Forgejo", Ok::<_, toml::de::Error>(ForgeType::Forgejo))]
        fn parse_forge_types(
            #[case] input: &str,
            #[case] expect: Result<ForgeType, toml::de::Error>,
        ) -> Result<()> {
            let toml = format!("value = \"{input}\"");
            let parsed: toml::Value = toml.parse()?;
            let result: Result<ForgeType, toml::de::Error> = parsed["value"].clone().try_into();

            assert_eq!(result, expect, "{input}");

            Ok(())
        }
    }
}
