//
use std::path::PathBuf;

use derive_more::with_trait::Deref;

/// The path to the directory containing the git repository.
#[derive(
    Clone,
    Debug,
    derive_more::From,
    PartialEq,
    Eq,
    PartialOrd,
    Ord,
    derive_more::AsRef,
    derive_more::Constructor,
)]
pub struct GitDir {
    pathbuf: PathBuf,
}
impl GitDir {
    pub(crate) const fn pathbuf(&self) -> &PathBuf {
        &self.pathbuf
    }

    pub(crate) fn as_fs(&self) -> kxio::fs::FileSystem {
        kxio::fs::new(PathBuf::clone(Self::pathbuf(self)))
    }
}
impl std::fmt::Display for GitDir {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "{}", &self.pathbuf.display())
    }
}
impl Deref for GitDir {
    type Target = PathBuf;

    fn deref(&self) -> &Self::Target {
        &self.pathbuf
    }
}
impl From<&GitDir> for PathBuf {
    fn from(value: &GitDir) -> Self {
        value.to_path_buf()
    }
}
impl From<&GitDir> for kxio::fs::FileSystem {
    fn from(gitdir: &GitDir) -> Self {
        gitdir.as_fs()
    }
}
