//
use crate::core::config::{BranchName, RepoBranches, RepoConfig, RepoConfigSource, RepoPath};
use crate::prelude::*;

/// Defines a Repo within a `ForgeConfig` to be monitored by the server
/// Maps from `git-next-server.toml` at `forge.{forge}.repos.{name}`
#[derive(
    Builder,
    Clone,
    Debug,
    derive_more::From,
    PartialEq,
    Eq,
    PartialOrd,
    Ord,
    serde::Serialize,
    serde::Deserialize,
    derive_more::Display,
)]
#[display("{}@{}", repo, branch)]
pub struct ServerRepoConfig {
    repo: String,
    branch: String,
    main: Option<String>,
    next: Option<String>,
    dev: Option<String>,
}
impl ServerRepoConfig {
    pub(crate) fn repo(&self) -> RepoPath {
        RepoPath::new(self.repo.clone())
    }

    pub(crate) fn branch(&self) -> BranchName {
        BranchName::new(&self.branch)
    }

    /// Returns a `RepoConfig` from the server configuration if ALL THREE branches were provided
    #[must_use]
    pub fn repo_config(&self) -> Option<RepoConfig> {
        match (&self.main, &self.next, &self.dev) {
            (Some(main), Some(next), Some(dev)) => Some(RepoConfig::new(
                RepoBranches::new(s!(main), s!(next), s!(dev)),
                RepoConfigSource::Server,
            )),
            _ => None,
        }
    }
}
