//
use crate::core::git::{ForgeLike, RepoDetails};

#[cfg(feature = "forge-forgejo")]
use crate::forges::forgejo::Forgejo;

#[cfg(feature = "forge-github")]
use crate::forges::github::Github;

use kxio::net::Net;

#[derive(Clone, Debug)]
pub struct Forge;

impl Forge {
    pub fn create(repo_details: RepoDetails, net: Net) -> Box<dyn ForgeLike> {
        match repo_details.forge.forge_type() {
            #[cfg(feature = "forge-forgejo")]
            crate::core::ForgeType::Forgejo => Box::new(Forgejo::new(repo_details, net)),
            #[cfg(feature = "forge-github")]
            crate::core::ForgeType::GitHub => Box::new(Github::new(repo_details, net)),
            _ => {
                drop(repo_details);
                drop(net);
                unreachable!();
            }
        }
    }
}

#[cfg(test)]
pub mod tests;
