//
use crate::core::{git, WebhookId};
use crate::prelude::*;

use kxio::net::Net;
use secrecy::ExposeSecret as _;

pub async fn unregister(
    webhook_id: &WebhookId,
    repo_details: &git::RepoDetails,
    net: &Net,
) -> git::forge::webhook::WebhookResult<()> {
    let hostname = &repo_details.forge.hostname();
    let repo_path = &repo_details.repo_path;
    let token = repo_details.forge.token().expose_secret();
    match net
        .delete(format!(
            "https://{hostname}/api/v1/repos/{repo_path}/hooks/{webhook_id}?token={token}"
        ))
        .send()
        .await
    {
        Err(e) => {
            warn!("Failed to unregister webhook");
            Err(git::forge::webhook::WebhookError::FailedToUnregister(
                e.to_string(),
            ))
        }
        _ => Ok(()),
    }
}
